/*
 * Copyright 2005-2010 the original author or authors.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wamblee.inject;


/**
 * The main entry point for programmatic dependency injection. A different
 * {@link InjectorFactory} can be plugged in for testing.
 * 
 * <p>
 * Given the following class:
 * </p>
 * <pre>
 *   class Pojo {
 *     &#064;EJB
 *     private Service service; 
 *   
 *     ...
 *   }
 * </pre>
 * 
 * injecting the EJB into a POJO is accomplished as follows:
 * 
 * <pre>
 * Pojo pojo = new Pojo();
 * SimpleInjector injector = new SimpleInjector(InjectorFactoryBuilder
 *     .getInjectorFactory());
 * injector.inject(pojo);
 * </pre>
 * 
 * Of course, the above example assumes the the injector understands the
 * &#064;EJB annotation (which of course CDI does).
 * 
 * The <code>SimpleInjector</code> should be cached. This is because the
 * <code>SimpleInjector</code> caches the {@link Injector} objects that it uses
 * internally for performance. This is done because creation of these internal
 * <code>Injector</code> objects may be costly. Caching the simple injector makes sure
 * that a class is not analysed again for annotations every time injection is
 * used.
 * 
 * For more advanced cases, the injector factory can also be directly
 * constructed instead of being obtained through the
 * {@link InjectorFactoryBuilder}.
 * 
 * @author Erik Brakkee
 */
public class SimpleInjector {

    private InjectorCache cache;

    /**
     * Constructs the injector.
     * 
     * @param aFactory
     *            Factory to use.
     */
    public SimpleInjector(InjectorFactory aFactory) {
        cache = new InjectorCache(aFactory);
    }

    /**
     * Injects into a given object.
     * 
     * @param aObject
     *            Object to inject into.
     */
    public void inject(Object aObject) {
        cache.getInjector(aObject.getClass()).inject(aObject);
    }
}
